## Example code for indicator analysis, to be run in the R Software environment
## Download from https://www.r-project.org/

## The indicator utilised is this example is: 'The Number of children referred to the Children’s 
## Reporter on offence grounds per 10,000 children', provided by the Scottish Children's Reporter 
## Administration (SCRA) 

# Install necessary packages
library(tidyverse); library(lubridate);library(reshape);
library(lme4); library(car); library(pbkrtest)


### Set the working directory of R to be the folder where the code and data are shared 
setwd("ADD DIRECTORY HERE")

## Read in the overdispersion function that will be required later
source("overdispersion_fun.r")


## Set up colours used for plotting
colours_vec <- c("#008080", "#768692", "#EA7600", "#A51890","#0077C8", "#658D1B")


## Read in  and examine indicator data (i.e. outcome variable)

indicator <- read.csv("Example_Indicator_CSR.csv", skip=5)
head(indicator) 


## Provide start year of time series and number of years available
start_year <- 2011
no_years <- 11


## Specify column names
colnames(indicator) <- c("LA", start_year:(start_year+(no_years-1)))
str(indicator)

## Convert indicator data to long format for modelling
ind_long <- melt(indicator)
names(ind_long) <- c("LA", "Year", "Indicator_value")
head(ind_long)


## Read in mid year population estimates to convert indicator to rates per 10,000 - using 0-17 population.
mid_year_pops <- read.csv("Mid-year-pops-0-17.csv", skip=3)
colnames(mid_year_pops) <- substr(colnames(mid_year_pops), 2, 5)

##Extract relevant portion of the mid year pops table (i.e. relevant years)
mid_years_needed <- mid_year_pops[1:32,colnames(mid_year_pops) %in% colnames(indicator)]


## Read in data on contextual factors (i.e. explanatory variables)
context_info <- read.csv("contextual_factors.csv", skip=3)[1:32,]
head(context_info)

##Convert HSCP start dates to year format
context_info$HSCP_start <- year(parse_date_time(context_info$HSCP_start, orders=c("dmy", "my", "Y")))


## Convert contextual info to long format for modelling
context_long <- reshape(context_info, idvar=c("LA", "Int_level", "Coterminous", "HSCP_start"), direction="long", 
                        varying=list(SIMD=grep("SIMD", names(context_info)), PopDens=grep("PopDens", names(context_info))),
                        v.names = c("SIMD_value", "PopDens") )

head(context_long, 40)
head(ind_long, 40)

# Remove columns with 'time' and 'LA' before combing with indicator data
context_long <- context_long[,-c(grep("time", names(context_long)), grep("LA", names(context_long)))]


## Add variable to depict which measurements were affected by the COVID-19 pandemic 
## (start date taken to be 23/3/20)
## This variable is tailored to the reporting period for each indicator, 
## e.g. for a given year, value of variable = no. of months that year that were during the pandemic/12. 
## The reporting period for this indicator
## is Apr-March, so none of the year 2019-20 was during the pandemic (with the exception of the final week). 
## Conversely, the full year reporting year 2020-21 was during the pandemic. (i.e. 12/12 months = 1)

covid_ind <- rep(0, length(ind_long$Year))
covid_ind[ind_long$Year == 2020] <- 0 #(will sometimes be 1/3 for August-July periods, e.g.)
covid_ind[ind_long$Year == 2021] <- 1
context_long <- cbind(context_long, covid_ind)

###Create one dataset containing all data

final_data <- cbind(ind_long, context_long)
str(final_data)

## Convert columns into the necessary data format
final_data$Int_level <- factor(final_data$Int_level, levels=c("No structural integration", "Partial structural integration", "Full structural integration"))
final_data$Year <- as.numeric(as.character(final_data$Year))
final_data$Coterminous <- factor(final_data$Coterminous)
final_data$LA <- factor(final_data$LA)

#Create variable for time as year relative to integration
final_data$year_rel <- final_data$Year - final_data$HSCP_start

## Add indicator to specify whether each datapoint was measured pre- or post- integration
final_data$pre_or_post <- rep("Pre_Int", length(final_data[,1]))
final_data$pre_or_post[final_data$year_rel>0] <- "Post_Int"
final_data$pre_or_post <- factor(final_data$pre_or_post, levels=c("Pre_Int", "Post_Int"))

## Scale numerical variables to improve model convergence

final_data$PopDens <- scale(final_data$PopDens)
final_data$SIMD_value <- scale(final_data$SIMD_value)
final_data$Year2 <- scale(final_data$Year)


## Add in mid year population data to calculate indicator values as a rate per 10,000
final_data$mid_year_pops <- melt(mid_years_needed)[,2]
final_data$rate <- final_data$Indicator_value/final_data$mid_year_pops*10000

## Calculate the log population per 10,000 for use as an offset in the model
final_data$ln_pops <- log(final_data$mid_year_pops/10000)


## Set up matrix to plot model estimates for the pre- and post-integration trends, using 2016 as the cutoff 
postPredict <- expand.grid (Int_level=unique(final_data$Int_level), pre_or_post="Post_Int", 
                            Year2=unique(final_data$Year2[final_data$Year>=2016]),
                            SIMD_value=mean(final_data$SIMD_value),
                            covid_ind=0, Coterminous="No", PopDens=mean(final_data$PopDens), 
                            ln_pops=mean(final_data$ln_pops))
postPredict$Year <- rep(2016:2021, each=3)
prePredict <- expand.grid (Int_level=unique(final_data$Int_level), pre_or_post="Pre_Int", 
                           Year2=unique(final_data$Year2[final_data$Year<=2016]),
                           SIMD_value=mean(final_data$SIMD_value),
                           covid_ind=0, Coterminous="No", PopDens=mean(final_data$PopDens), 
                           ln_pops=mean(final_data$ln_pops))
prePredict$Year <- rep(start_year:2016, each=3)

## Set covid indicator years for plot

postPredict$covid_ind[postPredict$Year==2021] <- 1

## Descriptive Plots of raw data

##Add or remove 'per 10,000 to the plot as required##
ggplot(data=final_data, aes(x=Year, y=rate, colour=LA)) +  
  geom_line() +   #facet_wrap(~Int_level) + # Remove '#' before facet_wrap to see each integration level separately
  labs(x="Year", y="Number of Children per 10,000") +   theme(text = element_text(size = 15)) + theme(legend.position = "bottom")

## Interaction plot (ignores correlated data structure so not definitive, but provides an idea of trend.) 
interaction.plot(final_data$Year, final_data$Int_level, final_data$rate, lwd=2, col=1:3)
abline(v=6, lty=2, col=2)

##Plot data by integration level (ignores correlated data structure so not definitive, but provides an idea of trend.) 
# Can run without the facet wrap to see combined trend for all data
ggplot(data=final_data, aes(x=Year, y=rate)) + geom_point() + geom_smooth() + facet_wrap(~Int_level) 


# #### Check whether a poisson or neg binomial model is more appropriate for the data (based on overdispersion) 
# Offset included in the model to convert number of children to a rate per 10,000

# ##Poisson model
model.pois <- glmer(Indicator_value ~ offset(ln_pops) +
                       Year2*pre_or_post + Int_level*pre_or_post +
                       covid_ind + Coterminous +
                       SIMD_value + PopDens +
                       (1|LA), data=final_data, family= poisson(link="log"),
                     na.action=na.exclude)

summary(model.pois)
overdisp_fun(model.pois) # ratio = 6.3 -  strong evidence of overdispersion so fit negative binomial.

## Negative binomial model
model.nb <- glmer.nb(Indicator_value ~ offset(ln_pops) +
                        Year2*pre_or_post + Int_level*pre_or_post +
                        covid_ind +
                        Coterminous +
                        SIMD_value +
                        PopDens +
                        (1|LA), data=final_data, na.action=na.exclude, nAGQ=0)
## Note - Fit is almost identical with nAGQ=0 but much faster, used to decrease
## computing time for PBmodcomp 

summary(model.nb)
overdisp_fun(model.nb) # dispersion ratio = 0.90 -  better. 
## Negative binomial to be used as most appropriate distribution for this data


###### EXPLORE THE NATIONAL TREND OVER THE FULL TIME PERIOD ####

mean(final_data$rate)
sd(final_data$rate)

## Calculate one standard deviation above and below for plot in Table X
mean(final_data$rate) - sd(final_data$rate); mean(final_data$rate) +sd(final_data$rate)


## National trend calculated from raw data
mid_years_needed_Scot <- mid_year_pops[33,colnames(mid_year_pops) %in% colnames(indicator)]
national_total <- apply(indicator[,2:(no_years+1)], 2, sum)
national_trend <- national_total/mid_years_needed_Scot*10000; national_trend


## National trend estimated by model with time as continuous to determine direction of overall trend
modelTrend <- glmer.nb(Indicator_value ~ offset(ln_pops) +
                         Year +
                         (1|LA), data=final_data, na.action=na.exclude, nAGQ=0)
summary(modelTrend)
## Calculate confidence intervals for estimates, set seed so simulations are reproducible
set.seed(17)
confint(modelTrend, method="boot", nsim=1000)

## Calculate trend estimate from model
predictYears <- expand.grid(Year=unique(final_data$Year), ln_pops=mean(final_data$ln_pops))
modelledTrend <- predict(modelTrend, newdata=predictYears, re.form=~0, type="response")/exp(mean(final_data$ln_pops))

## Plot calculate and modelled trend
col_vec <- c("black", "#cf3e96") # hot pink 
pred_to_plot <- data.frame(Year=rep(names(national_trend), times=2),
                           value=unlist(c(modelledTrend, national_trend)), type=rep(c("Modelled Trend", "National Average"), each=11))
pred_to_plot$Year <- as.numeric(pred_to_plot$Year)

ggplot(data=final_data, aes(x=Year, y=rate), colour="lightgrey") + 
  geom_line(aes(group=LA),linewidth=0.5, alpha=0.3, linetype=2) +  
  geom_line(data=pred_to_plot, aes(x=Year, y=value, colour=type), linewidth=1.5) +
  labs(title="National trend: Actual vs modelled", 
       subtitle="Number of children referred to Children’s Reporter on offence grounds \nper 10,000 children",
       y="Number per 10,000 children", colour="")  +
  scale_x_continuous(breaks=c(start_year:(start_year+no_years-1))) + scale_colour_manual(values=col_vec[c(2,1)]) + 
  theme_classic() + theme(text = element_text(size = 20)) 


###### FIT MODELS TO EXPLORE THE IMPACTS OF INTEGRATION ####

## MODEL A0 - Two slopes model with covariates added and NO integration

modelA0 <- glmer.nb(Indicator_value ~ offset(ln_pops) +
                     Year2*pre_or_post +
                     covid_ind + 
                     Coterminous + 
                     SIMD_value + 
                     PopDens +
                     (1|LA), data=final_data, na.action=na.exclude, nAGQ=0)


summary(modelA0)

## Plot model A0 estimates
postfits_noRE <- predict(modelA0, newdata=postPredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))
prefits_noRE <- predict(modelA0, newdata=prePredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))

ggplot(data=final_data, aes(x=Year, y=rate), colour="lightgrey") + 
  geom_line(aes(group=LA),linewidth=0.5, alpha=0.3, linetype=2) +  
  geom_line(data=postPredict, aes(x=Year, y=postfits_noRE), linewidth=1.5, colour="black") +
  geom_line(data=prePredict, aes(x=Year, y=prefits_noRE), linewidth=1.5, colour="black") + 
  geom_vline(xintercept = 2016, linetype=2) +
  labs(title="Model A0 - No group effect", colour="Integration Level", 
       y="Number per 10,000 children") + 
  scale_colour_manual(values=colours_vec[c(1,3,4)]) +
  scale_x_continuous(breaks=c(start_year:(start_year+no_years-1))) +
  theme_classic() + theme(text = element_text(size = 20))

## MODEL A1 - Integration as fixed effect (same pre- vs post-)

modelA1 <- glmer.nb(Indicator_value ~ offset(ln_pops) + Int_level +
                     Year2*pre_or_post +
                     covid_ind + 
                     Coterminous + 
                     SIMD_value + 
                     PopDens +
                     (1|LA), data=final_data, na.action=na.exclude, nAGQ=0)
summary(modelA1)


## Plot model A1 estimates
postfits_noRE <- predict(modelA1, newdata=postPredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))
prefits_noRE <- predict(modelA1, newdata=prePredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))

ggplot(data=final_data, aes(x=Year, y=rate, colour=Int_level)) + 
  geom_line(aes(group=LA),linewidth=0.5, alpha=0.5, linetype=2) +  
  geom_line(data=postPredict, aes(x=Year, y=postfits_noRE), linewidth=1.5, linetype=rep(c(1, 1, 2), each=6)) +
  geom_line(data=prePredict, aes(x=Year, y=prefits_noRE), linewidth=1.5, linetype=rep(c(1, 1, 2), each=6)) + 
  geom_vline(xintercept = 2016, linetype=2) +
  labs(title="Model A1 - Group effect constant over time", colour="Integration level", 
       y="Number per 10,000 children") + 
  scale_colour_manual(values=colours_vec[c(4,1,3)]) +
  scale_x_continuous(breaks=c(start_year:(start_year+no_years-1))) +
  theme_classic() + theme(text = element_text(size = 20))

## Comparison of Model A0 and Model A1, set seed so simulations are reproducible
set.seed(241)
PBmodcomp(modelA1, modelA0, nsim=1000) # p-value = 0.98 

## MODEL A2 - Integration as interaction effect (different pre- vs post-)

modelA2 <- glmer.nb(Indicator_value ~ offset(ln_pops) +
                     Year2*pre_or_post + Int_level*pre_or_post +
                     covid_ind + 
                     Coterminous + 
                     SIMD_value + 
                     PopDens +
                     (1|LA), data=final_data, na.action=na.exclude, nAGQ=0)
summary(modelA2)

## Plot model A2 estimates
postfits_noRE <- predict(modelA2, newdata=postPredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))
prefits_noRE <- predict(modelA2, newdata=prePredict, re.form=~0, type="response")/exp(mean(final_data$ln_pops))

##Plotting pre- and post- lines that both hit 0

ggplot(data=final_data, aes(x=Year, y=rate, colour=Int_level)) + 
  geom_line(aes(group=LA),linewidth=0.5, alpha=0.5, linetype=2) +  
  geom_line(data=postPredict, aes(x=Year, y=postfits_noRE), linewidth=1.5, linetype=rep(c(1, 1, 1), each=6)) +
  geom_line(data=prePredict, aes(x=Year, y=prefits_noRE), linewidth=1.5, linetype=rep(c(1, 2, 1), each=6)) + 
  geom_vline(xintercept = 2016, linetype=2) +
  labs(title="Model A2 - Group effect can differ pre- vs post-integration", colour="Integration level", 
       y="Number per 10,000 children") + 
  scale_colour_manual(values=colours_vec[c(4,1,3)]) +
  scale_x_continuous(breaks=c(start_year:(start_year+no_years-1))) +
  theme_classic() + theme(text = element_text(size = 20)) 

## Comparison of Model A1  and model A2, set seed so simulations are reproducible
set.seed(6413)
PBmodcomp(modelA2, modelA1, nsim=1000) # p-value = 0.147 



#### ADJUSTING FOR MULTIPLE COMPARISONS #####

## After p-values had been calculated for the necessary model comparison for all 25 indicators, 
## they were stored in a vector named 'p_vals' and
## adjusted for multiple comparisons using the function below

p.adjust(p_vals, method="BH")



### CHECK WHETHER HIGHLAND (WHICH IS SOMEWHAT OF AN OUTLIER) HAS A SUBSTANTIAL EFFECT ###

### To obtain a p-value with data from Highland removed from the analysis, execute the two lines  below (after
## removing the hash symbols) then
## re-run the above code from 'Descriptive Plots of raw data' down ###

#highland_ind <- which(final_data$LA=="Highland")
#final_data <- final_data[-highland_ind,]


### CHECKING MODEL ASSUMPTIONS ####

#### Check model assumptions for Model C to ensure it is a suitable
## model and it is appropriate to draw inferences 


###Plot of predicted values vs observed values - GOOD
plot(final_data$Indicator_value, predict(modelA2, type="response"))
abline(0,1)

## Check dispersion for best model (or full model) - only applicable to poisson/neg binom
overdisp_fun(modelA2) # ratio=0.9, so okay

### Check for multicollinearity in the explanatory variables

##Check for correlation of SIMD and POP dens
plot(context_long$PopDens, context_long$SIMD_value)
cor(context_long$PopDens, context_long$SIMD_value) ##r=.547 

## Confirm levels of correlation are not problematic through inspection of VIF (variance inflation factors)
vif(modelA2) ## All non-interaction terms under 3, indicating levels of correlation are not problematic.

## Check for normality of random effects

r_int<- ranef(modelA2)$LA
qqnorm(unlist(r_int))
qqline(unlist(r_int))
hist(unlist(r_int))
shapiro.test(unlist(r_int)) 
## Assumption okay. Shapiro test p-value > 0.05 (p=.21), implying that the distribution of the random effects are 
## not significantly different from normal distribution.


## Check for any residual autocorrelation
acf(residuals(modelA2, retype="normalized")) #Fine
pacf(residuals(modelA2, retype="normalized"))  #Fine


